@@ -18,13 +18,11 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program.  If not, you can find it on the World Wide
-# Web at http://www.gnu.org/copyleft/gpl.html, or write to the Free
-# Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
-# MA 02111-1307, USA.
+# along with this program. If not, see <http://www.gnu.org/licenses/>.
 
 use strict;
 use Getopt::Long;
+use File::Temp qw/tempfile/;
 
 sub init_hashes;
 
@@ -50,6 +48,17 @@ EOF
 
 my ($opt_echo, $opt_force, $opt_extra, $opt_posix);
 my ($opt_help, $opt_version);
+my @filenames;
+
+# Detect if STDIN is a pipe
+if (-p STDIN or -f STDIN) {
+    my ($tmp_fh, $tmp_filename) = tempfile("chkbashisms_tmp.XXXX", TMPDIR => 1, UNLINK => 1);
+    while (my $line = <STDIN>) {
+        print $tmp_fh $line;
+    }
+    close($tmp_fh);
+    push(@ARGV, $tmp_filename);
+}
 
 ##
 ## handle command-line options
@@ -80,31 +89,39 @@ init_hashes;
 foreach my $filename (@ARGV) {
     my $check_lines_count = -1;
 
+    my $display_filename = $filename;
+    if ($filename =~ /chkbashisms_tmp\.....$/) {
+        $display_filename = "(stdin)";
+    }
+
     if (!$opt_force) {
 	$check_lines_count = script_is_evil_and_wrong($filename);
     }
 
     if ($check_lines_count == 0 or $check_lines_count == 1) {
-	warn "script $filename does not appear to be a /bin/sh script; skipping\n";
+	warn "script $display_filename does not appear to be a /bin/sh script; skipping\n";
 	next;
     }
 
     if ($check_lines_count != -1) {
-	warn "script $filename appears to be a shell wrapper; only checking the first "
+	warn "script $display_filename appears to be a shell wrapper; only checking the first "
 	     . "$check_lines_count lines\n";
     }
 
     unless (open C, '<', $filename) {
-	warn "cannot open script $filename for reading: $!\n";
+	warn "cannot open script $display_filename for reading: $!\n";
 	$status |= 2;
 	next;
     }
 
     my $cat_string = "";
+    my $cat_indented = 0;
     my $quote_string = "";
     my $last_continued = 0;
     my $continued = 0;
     my $found_rules = 0;
+    my $buffered_orig_line = "";
+    my $buffered_line = "";
 
     while (<C>) {
 	next unless ($check_lines_count == -1 or $. <= $check_lines_count);
@@ -123,18 +140,18 @@ foreach my $filename (@ARGV) {
 		next if $opt_force;
 
 		if ($interpreter =~ m,/bash$,) {
-		    warn "script $filename is already a bash script; skipping\n";
+		    warn "script $display_filename is already a bash script; skipping\n";
 		    $status |= 2;
 		    last;  # end this file
 		}
 		elsif ($interpreter !~ m,/(sh|posh)$,) {
 ### ksh/zsh?
-		    warn "script $filename does not appear to be a /bin/sh script; skipping\n";
+		    warn "script $display_filename does not appear to be a /bin/sh script; skipping\n";
 		    $status |= 2;
 		    last;
 		}
 	    } else {
-		warn "script $filename does not appear to have a \#! interpreter line;\nyou may get strange results\n";
+		warn "script $display_filename does not appear to have a \#! interpreter line;\nyou may get strange results\n";
 	    }
 	}
 
@@ -153,7 +170,10 @@ foreach my $filename (@ARGV) {
 	# will be treated as part of the comment.
 	# s/^(?:.*?[^\\])?$quote_string(.*)$/$1/ if $quote_string ne "";
 
-	next if m,^\s*\#,;  # skip comment lines
+	# skip comment lines
+	if (m,^\s*\#, && $quote_string eq '' && $buffered_line eq '' && $cat_string eq '') {
+	    next;
+	}
 
 	# Remove quoted strings so we can more easily ignore comments
 	# inside them
@@ -170,6 +190,21 @@ foreach my $filename (@ARGV) {
 	    $_ = $orig_line;
 	}
 
+	# Handle line continuation
+	if (!$makefile && $cat_string eq '' && m/\\$/) {
+	    chop;
+	    $buffered_line .= $_;
+	    $buffered_orig_line .= $orig_line . "\n";
+	    next;
+	}
+
+	if ($buffered_line ne '') {
+	    $_ = $buffered_line . $_;
+	    $orig_line = $buffered_orig_line . $orig_line;
+	    $buffered_line ='';
+	    $buffered_orig_line ='';
+	}
+
 	if ($makefile) {
 	    $last_continued = $continued;
 	    if (/[^\\]\\$/) {
@@ -185,14 +220,17 @@ foreach my $filename (@ARGV) {
 		$_ = $1 if $1;
 	    } 
 
-	    last if m%^(export )?SHELL\s*:?=\s*(/bin/)?bash\s*%;
+	    last if m%^\s*(override\s|export\s)?\s*SHELL\s*:?=\s*(/bin/)?bash\s*%;
 
+	    # Remove "simple" target names
+	    s/^[\w%.-]+(?:\s+[\w%.-]+)*::?//;
 	    s/^\t//;
-	    s/(\$){2}/$1/;
-	    s/^[\s\t]*@//;
+	    s/(?<!\$)\$\((\w+)\)/\${$1}/g;
+	    s/(\$){2}/$1/g;
+	    s/^[\s\t]*[@-]{1,2}//;
 	}
 
-	if ($cat_string ne "" and m/^\Q$cat_string\E$/) {
+	if ($cat_string ne "" && (m/^\Q$cat_string\E$/ || ($cat_indented && m/^\t*\Q$cat_string\E$/))) {
 	    $cat_string = "";
 	    next;
 	}
@@ -257,7 +295,7 @@ foreach my $filename (@ARGV) {
 		    my $otherquote = ($quote eq "\"" ? "\'" : "\"");
 
 		    # Remove balanced quotes and their content
-		    $templine =~ s/(^|[^\\\"](?:\\\\)*)\'(?:\\.|[^\\\'])+\'/$1/g;
+		    $templine =~ s/(^|[^\\\"](?:\\\\)*)\'[^\']*\'/$1/g;
 		    $templine =~ s/(^|[^\\\'](?:\\\\)*)\"(?:\\.|[^\\\"])+\"/$1/g;
 
 		    # Don't flag quotes that are themselves quoted
@@ -265,6 +303,8 @@ foreach my $filename (@ARGV) {
 		    $templine =~ s/$otherquote.*?$quote.*?$otherquote//g;
 		    # "\""
 		    $templine =~ s/(^|[^\\])$quote\\$quote$quote/$1/g;
+		    # \' or \"
+		    $templine =~ s/\\[\'\"]//g;
 		    my $count = () = $templine =~ /(^|(?!\\))$quote/g;
 
 		    # If there's an odd number of non-escaped
@@ -291,7 +331,7 @@ foreach my $filename (@ARGV) {
 		    $found = 1;
 		    $match = $1;
 		    $explanation = "sourced script with arguments";
-		    output_explanation($filename, $orig_line, $explanation);
+		    output_explanation($display_filename, $orig_line, $explanation);
 		}
 	    }
 
@@ -307,7 +347,7 @@ foreach my $filename (@ARGV) {
 		    $found = 1;
 		    $match = $1;
 		    $explanation = $expl;
-		    output_explanation($filename, $orig_line, $explanation);
+		    output_explanation($display_filename, $orig_line, $explanation);
 		}
 	    }
 
@@ -315,7 +355,7 @@ foreach my $filename (@ARGV) {
 	    if ($line =~ m/(.*)($re)/){
 		my $count = () = $1 =~ /(^|[^\\])\'/g;
 		if( $count % 2 == 0 ) {
-		    output_explanation($filename, $orig_line, q<$'...' should be "$(printf '...')">);
+		    output_explanation($display_filename, $orig_line, q<$'...' should be "$(printf '...')">);
 		}
 	    }   
 
@@ -341,7 +381,7 @@ foreach my $filename (@ARGV) {
 	    if ($line =~ m/(.*)($re)/){
 		my $count = () = $1 =~ /(^|[^\\])\"/g;
 		if( $count % 2 == 0 ) {
-		    output_explanation($filename, $orig_line, q<$"foo" should be eval_gettext "foo">);
+		    output_explanation($display_filename, $orig_line, q<$"foo" should be eval_gettext "foo">);
 		}
 	    }   
 
@@ -350,7 +390,7 @@ foreach my $filename (@ARGV) {
 		    $found = 1;
 		    $match = $1;
 		    $explanation = $expl;
-		    output_explanation($filename, $orig_line, $explanation);
+		    output_explanation($display_filename, $orig_line, $explanation);
 		}
 	    }
 
@@ -363,19 +403,27 @@ foreach my $filename (@ARGV) {
 		    $found = 1;
 		    $match = $1;
 		    $explanation = $expl;
-		    output_explanation($filename, $orig_line, $explanation);
+		    output_explanation($display_filename, $orig_line, $explanation);
 		}
 	    }
 
 	    # Only look for the beginning of a heredoc here, after we've
 	    # stripped out quoted material, to avoid false positives.
-	    if ($cat_line =~ m/(?:^|[^<])\<\<\-?\s*(?:[\\]?(\w+)|[\'\"](.*?)[\'\"])/) {
-		$cat_string = $1;
-		$cat_string = $2 if not defined $cat_string;
+	    if ($cat_line =~ m/(?:^|[^<])\<\<(\-?)\s*(?:[\\]?(\w+)|[\'\"](.*?)[\'\"])/) {
+		$cat_indented = ($1 && $1 eq '-')? 1 : 0;
+		$cat_string = $2;
+		$cat_string = $3 if not defined $cat_string;
             }
 	}
     }
 
+    warn "error: $filename:  Unterminated heredoc found, EOF reached. Wanted: <$cat_string>\n"
+	if ($cat_string ne '');
+    warn "error: $filename: Unterminated quoted string found, EOF reached. Wanted: <$quote_string>\n"
+	if ($quote_string ne '');
+    warn "error: $filename: EOF reached while on line continuation.\n"
+	if ($buffered_line ne '');
+
     close C;
 }
 
@@ -481,10 +529,8 @@ sub init_hashes {
 	qr'(?<![\$\(])\(\(.*\)\)' =>     q<'((' should be '$(('>,
 	qr'(?:^|\s+)(\[|test)\s+-a' =>            q<test with unary -a (should be -e)>,
 	qr'\&>' =>	               q<should be \>word 2\>&1>,
-	qr'(<\&|>\&)\s*((-|\d+)[^\s;|)`&\\\\]|[^-\d\s]+)' =>
+	qr'(<\&|>\&)\s*((-|\d+)[^\s;|)}`&\\\\]|[^-\d\s]+(?<!\$)(?!\d))' =>
 				       q<should be \>word 2\>&1>,
-	$LEADIN . qr'kill\s+-[^sl]\w*' => q<kill -[0-9] or -[A-Z]>,
-	$LEADIN . qr'trap\s+["\']?.*["\']?\s+.*[1-9]' => q<trap with signal numbers>,
 	qr'\[\[(?!:)' => q<alternative test command ([[ foo ]] should be [ foo ])>,
 	qr'/dev/(tcp|udp)'	    => q</dev/(tcp|udp)>,
 	$LEADIN . qr'builtin\s' =>        q<builtin>,
@@ -516,6 +562,13 @@ sub init_hashes {
 	$LEADIN . qr'(sh|\$\{?SHELL\}?) -[rD]' => q<sh -[rD]>,
 	$LEADIN . qr'(sh|\$\{?SHELL\}?) --\w+' =>  q<sh --long-option>,
 	$LEADIN . qr'(sh|\$\{?SHELL\}?) [-+]O' =>  q<sh [-+]O>,
+	qr'\[\^[^]]+\]' =>  q<[^] should be [!]>,
+	$LEADIN . qr'printf\s+-v' => q<'printf -v var ...' should be var='$(printf ...)'>,
+	$LEADIN . qr'coproc\s' =>        q<coproc>,
+	qr';;?&' =>  q<;;& and ;& special case operators>,
+	$LEADIN . qr'jobs\s' =>  q<jobs>,
+#	$LEADIN . qr'jobs\s+-[^lp]\s' =>  q<'jobs' with option other than -l or -p>,
+	$LEADIN . qr'command\s+-[^p]\s' =>  q<'command' with option other than -p>,
     );
 
     %string_bashisms = (
@@ -538,6 +591,12 @@ sub init_hashes {
 	qr'\$\{?SHLVL\}?\b'           => q<$SHLVL>,
 	qr'<<<'                       => q<\<\<\< here string>,
 	$LEADIN . qr'echo\s+(?:-[^e\s]+\s+)?\"[^\"]*(\\[abcEfnrtv0])+.*?[\"]' => q<unsafe echo with backslash>,
+	qr'\$\(\([\s\w$*/+-]*\w\+\+.*?\)\)'   => q<'$((n++))' should be '$n; $((n=n+1))'>,
+	qr'\$\(\([\s\w$*/+-]*\+\+\w.*?\)\)'   => q<'$((++n))' should be '$((n=n+1))'>,
+	qr'\$\(\([\s\w$*/+-]*\w\-\-.*?\)\)'   => q<'$((n--))' should be '$n; $((n=n-1))'>,
+	qr'\$\(\([\s\w$*/+-]*\-\-\w.*?\)\)'   => q<'$((--n))' should be '$((n=n-1))'>,
+	qr'\$\(\([\s\w$*/+-]*\*\*.*?\)\)'   => q<exponentiation is not POSIX>,
+	$LEADIN . qr'printf\s["\'][^"\']+?%[qb].+?["\']' => q<printf %q|%b>,
     );
 
     %singlequote_bashisms = (
@@ -554,6 +613,8 @@ sub init_hashes {
 	$bashisms{$LEADIN . qr'local\s+\w+='} = q<local foo=bar>;
 	$bashisms{$LEADIN . qr'local\s+\w+\s+\w+'} = q<local x y>;
 	$bashisms{$LEADIN . qr'((?:test|\[)\s+.+\s-[ao])\s'} = q<test -a/-o>;
+	$bashisms{$LEADIN . qr'kill\s+-[^sl]\w*'} = q<kill -[0-9] or -[A-Z]>;
+	$bashisms{$LEADIN . qr'trap\s+["\']?.*["\']?\s+.*[1-9]'} = q<trap with signal numbers>;
     }
 
     if ($makefile) {
