@@ -16,8 +16,7 @@
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# with this program. If not, see <http://www.gnu.org/licenses/>.
 
 =head1 NAME
 
@@ -25,11 +24,11 @@ licensecheck - simple license checker for source files
 
 =head1 SYNOPSIS
 
-B<licensecheck> B<--help|--version>
+B<licensecheck> B<--help>|B<--version>
 
 B<licensecheck> [B<--no-conf>] [B<--verbose>] [B<--copyright>]
-[B<-l|--lines=N>] [B<-i|--ignore=regex>] [B<-c|--check=regex>]
-[B<-r|--recursive>] I<list of files and directories to check>
+[B<-l>|B<--lines=>I<N>] [B<-i>|B<--ignore=>I<regex>] [B<-c>|B<--check=>I<regex>]
+[B<-r>|B<--recursive>] I<list of files and directories to check>
 
 =head1 DESCRIPTION
 
@@ -44,30 +43,30 @@ the files contained within to the list of files to process.
 
 =over 4
 
-=item B<--verbose> B<--no-verbose>
+=item B<--verbose>, B<--no-verbose>
 
 Specify whether to output the text being processed from each file before
 the corresponding license information.
 
 Default is to be quiet.
 
-=item B<-l=N> B<--lines=N>
+=item B<-l=>I<N>, B<--lines=>I<N>
 
 Specify the number of lines of each file's header which should be parsed 
 for license information. (Default is 60).
 
-=item B<-i=regex> B<--ignore=regex>
+=item B<-i=>I<regex>, B<--ignore=>I<regex>
 
 When processing the list of files and directories, the regular 
 expression specified by this option will be used to indicate those which 
 should not be considered (e.g. backup files, VCS metadata). 
 
-=item B<-r> B<--recursive>
+=item B<-r>, B<--recursive>
 
 Specify that the contents of directories should be added 
 recursively.
 
-=item B<-c=regex> B<--check=regex>
+=item B<-c=>I<regex>, B<--check=>I<regex>
 
 Specify a pattern against which filenames will be matched in order to 
 decide which files to check the license of.
@@ -78,7 +77,7 @@ The default includes common source files.
 
 Also display copyright text found within the file
 
-=item B<--no-conf> B<--noconf>
+=item B<--no-conf>, B<--noconf>
 
 Do not read any configuration files. This can only be used as the first
 option given on the command-line.
@@ -97,23 +96,23 @@ ignored for this purpose.  The currently recognised variables are:
 
 =item B<LICENSECHECK_VERBOSE>
 
-If this is set to I<yes>, then it is the same as the --verbose command
+If this is set to I<yes>, then it is the same as the B<--verbose> command
 line parameter being used. The default is I<no>.
 
 =item B<LICENSECHECK_PARSELINES>
 
 If this is set to a positive number then the specified number of lines
 at the start of each file will be read whilst attempting to determine
-the license(s) in use.  This is equivalent to the --lines command line
+the license(s) in use.  This is equivalent to the B<--lines> command line
 option.
 
 =back
 
 =head1 LICENSE
 
-This code is copyright by Adam D. Barratt <adam@adam-barratt.org.uk>, 
+This code is copyright by Adam D. Barratt <I<adam@adam-barratt.org.uk>>, 
 all rights reserved; based on a script of the same name from the KDE 
-SDK, which is copyright by <dfaure@kde.org>.
+SDK, which is copyright by <I<dfaure@kde.org>>.
 This program comes with ABSOLUTELY NO WARRANTY.
 You are free to redistribute this code under the terms of the GNU 
 General Public License, version 2 or later.
@@ -156,7 +155,7 @@ my $default_ignore_regex = '
 $default_ignore_regex =~ s/^#.*$//mg;
 $default_ignore_regex =~ s/\n//sg;
 
-my $default_check_regex = '\.(c(c|pp|xx)?|h(h|pp|xx)?|f(77|90)?|p(l|m)|xs|sh|php|py|rb|java|vala|el|sc(i|e)|cs)$';
+my $default_check_regex = '\.(c(c|pp|xx)?|h(h|pp|xx)?|f(77|90)?|p(l|m)|xs|sh|php|py|rb|java|vala|el|sc(i|e)|cs|pas|inc|dtd|xsl|mod)$';
 
 my $modified_conf_msg;
 
@@ -476,7 +475,7 @@ sub parselicense($) {
 	$license = "Perl $license";
     }
 
-    if ($licensetext =~ /under the Apache License, Version ([^ ]+) \(the License\)/) {
+    if ($licensetext =~ /under the Apache License, Version ([^ ]+)/) {
 	$license = "Apache (v$1) $license";
     }
 
